// Copyright 2006 FoxyTunes
// All Rights Reserved

function AlbumCoverCacheManager() {
    this._cacheDir = null;
    this._imagesDir = null;
    this._version = "1";
    this._unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
                                getService(Components.interfaces.nsIScriptableUnicodeConverter);

    this._unicodeConverter.charset = 'UTF-8';
}

AlbumCoverCacheManager.prototype = {

    init: function() {
	var dir = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("TmpD", Components.interfaces.nsIFile);
	dir.append("FoxyTunes");
	if( !dir.exists() || !dir.isDirectory() ) {
	    dir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
	}
	dir.append("CoverCache");
	if( !dir.exists() || !dir.isDirectory() ) {
	    dir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
	}

	this._cacheDir = dir;
	var imagesDir = dir.clone();
	imagesDir.append('images');
	if( !imagesDir.exists() || !imagesDir.isDirectory() ) {
	    imagesDir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
	}
	this._imagesDir = imagesDir;
    },    

    get: function(name, artist, imageSize, bIsTrack) {
	var fileName = this.generateCacheFileName(name, artist, imageSize, bIsTrack);
	var file = this._cacheDir.clone();
	file.append(fileName);
	if (!file.exists()) {
	    return null;
	}
	var data = this.readAsciFile(file);
	if (data == null || data.length == 0) {
	    return null;
	}
	var dataCacheObject = this.parseXMLCacheData(data);
	if (!dataCacheObject) {
	    return null;
	}
	// check expiration of the cache AND validation for the item cache version
	var d = new Date();
	var timeInMs = d.getTime();
	if ((timeInMs > dataCacheObject.expirationTimeInMs) ||
	     !this.isValidateCacheVersion(dataCacheObject.version)) {
	    try {
		file.remove(false);
	    } catch (e) {
		this.printLog("Failed to delete expired cache file: " + file.path + " - " + e);
	    }
	    return null;
	}

	var localImage = null;
	if (dataCacheObject.imageName != null) {
	    localImage = this.getLocalImageUri(dataCacheObject.imageName);
	    if (localImage == null) {
		try {
		    file.remove(false);
		} catch (e) {}
		return null;
	    }
	}
	var albumCoverData = new AlbumCoverData(localImage, dataCacheObject.link, dataCacheObject.height, dataCacheObject.width, dataCacheObject.title);
	return albumCoverData;
    },
    
    set: function(album, artist, track, imageSize, albumCoverData, daysToExpired, bytes, imageType ) {
	// get album cache filename if needed
	var albumCacheFileName = null;
	if (album) {
	    albumCacheFileName = this.generateCacheFileName(album, artist, imageSize, false);
	}

	// get track cache filename if needed
	var trackCacheFileName = null;
	if (track) {
	    trackCacheFileName = this.generateCacheFileName(track, artist, imageSize, true);
	} 

	var imageFileName = null;
	// we get bytes == null in case we want to cache 'not-found' result
	if (bytes != null ) {
	    // get cached image filename
	    var imageFileNameBase = albumCacheFileName ? albumCacheFileName : trackCacheFileName;
	    imageFileName = this.getImageNameByType(imageFileNameBase,  imageType);
	    if (!imageFileName) {
    		throw ('Faild to genarate image name');
	    }
	    var file = this._imagesDir.clone();
	    file.append(imageFileName);
	    if (!this.writeBinaryFile(file, bytes) ) {
	    	throw('Faild to write binary image');
	    }
	}

	var albumData;
	var trackData;
	if (albumCacheFileName) {
		albumData = this.saveXmlFlie(albumCoverData, albumCacheFileName, daysToExpired, imageFileName);
	}

	if (trackCacheFileName) {
		trackData = this.saveXmlFlie(albumCoverData, trackCacheFileName, daysToExpired, imageFileName);
	}

	return albumData ? albumData : trackData;
   },

    saveXmlFlie: function(albumCoverData, fileName, daysToExpired, imageName) {
	
	var data = this.generateXMLCacheData(albumCoverData, imageName, daysToExpired)
	//saving xml file
	var file = this._cacheDir.clone();
	file.append(fileName);
	if (!this.writeAsciFile(file, data) ) {
	    throw('Faild to save xml file');	
	}	

	// return the new cache data
	var localImage = null;
    if (imageName) {
        localImage = this.getLocalImageUri(imageName);
    }
	var albumCoverData = new AlbumCoverData(localImage, albumCoverData.link, albumCoverData.height, albumCoverData.width, albumCoverData.title);
	return albumCoverData;

    },

    getLocalImageUri: function(name) {
	var imageFile = this._imagesDir.clone();
	imageFile.append(name);

	if (!imageFile.exists() ) {
	    return null;
	}
	var ioService = Components.classes["@mozilla.org/network/io-service;1"]
		    .getService(Components.interfaces.nsIIOService);
	var localImage = ioService.newFileURI(imageFile);
	localImage = "file://" + localImage.path;
	return localImage;
    },

    generateXMLCacheData: function(albumCoverData, imageName, daysToExpired) {
	// clalculate expired
    	var minuteInMs = 1000*60;
	    var hourInMs = minuteInMs*60;
	    var dayInMs = hourInMs*24;
	    var d =  new Date();
	    var currentTimeInMs = d.getTime();
	    var expiration = currentTimeInMs + (daysToExpired * dayInMs);

        var data = '<?xml version="1.0" encoding="UTF-8"?>\n';
        data += '<image>\n';
	    data += '  <expiration>' + expiration + '</expiration>\n';
	    data += '  <version>' + this._version + '</version>\n';
	    data += '  <name>' + '<![CDATA[' + imageName + ']]>' + '</name>\n';
	    data += '  <height>' + albumCoverData.height + '</height>\n';
	    data += '  <width>' + albumCoverData.width + '</width>\n';
	    data += '  <link>' + '<![CDATA['+ albumCoverData.link + ']]>' + '</link>\n';
	    data += '  <title>' + '<![CDATA['+ this._unicodeConverter.ConvertFromUnicode(albumCoverData.title) + ']]>' + '</title>\n';
	    data += '</image>';
	    return data;	

    },
    
    encodeXMLChars: function(str) {
	var xmlStr = null;
	if (str) {
	    xmlStr = str.replace(/\&/g, "&amp;").replace(/\</g, "&lt;").replace(/\>/g, "&gt;").
			replace(/\"/g, "&quot;").replace(/\'/g, "&apos;")
	}
	return xmlStr;
    },

    parseXMLCacheData: function(data) {
	var parser = new DOMParser();
	var doc = parser.parseFromString(data,"text/xml");
	var dataCacheObject = new Object();

	dataCacheObject.expirationTimeInMs = this.getXMLElement(doc, "expiration");
	dataCacheObject.version            = this.getXMLElement(doc, "version");
	dataCacheObject.imageName          = this.getXMLElement(doc, "name");
	dataCacheObject.height             = this.getXMLElement(doc, "height");
	dataCacheObject.width              = this.getXMLElement(doc, "width");
	dataCacheObject.link               = this.getXMLElement(doc, "link");
	dataCacheObject.title               = this.getXMLElement(doc, "title");
    dataCacheObject.title               = this._unicodeConverter.ConvertToUnicode( this.getXMLElement(doc, "title") );
	return dataCacheObject;
    },
    
    getXMLElement: function(doc, tagName) {
	var elem = null;
	try {
	    elem = doc.getElementsByTagName(tagName)[0].firstChild.nodeValue;
	} catch (e) {
	    this.printLog("In xml file, failed to get element : " + tagName + " - " + e);
	    return null;
	}
	elem = (elem == "null") ? null : elem;
	return elem;
    },

    writeAsciFile: function(file, data) {
	var fileOutputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                         .createInstance(Components.interfaces.nsIFileOutputStream);
	try {
	    fileOutputStream.init(file, 0x02 | 0x08 | 0x20, 0777, 0); 
	    fileOutputStream.write(data, data.length);
	    fileOutputStream.close();
	} catch (e) {
	    this.printLog("Failed to write file: " + file.path + " - " + e);
	    try { 
		fileOutputStream.close(); 
	    } catch (e) {}
	    return false;
	}
	return true;
    },

    writeBinaryFile: function(file, bytes) {
	var fileOutputStream =  Components.classes["@mozilla.org/network/file-output-stream;1"].
                             createInstance(Components.interfaces.nsIFileOutputStream);      
	var binStream = Components.classes["@mozilla.org/binaryoutputstream;1"].
                             createInstance(Components.interfaces.nsIBinaryOutputStream);
	try {
	    fileOutputStream.init(file, 0x02 | 0x08 | 0x20, 0777, 0);
	    binStream.setOutputStream(fileOutputStream);
	    binStream.writeBytes(bytes, bytes.length);
	    binStream.close();
	    fileOutputStream.close();
	} catch (e) {
	    this.printLog("Failed to write binary file: " + file.path + " - " + e);
	    try { 
		binStream.close();
		fileOutputStream.close();
	    } catch (e) {}
	    return false;
	}
	return true;
    },

    getImageNameByType: function(fileName,imageType) {
        switch (imageType) {
	    case 'image/jpeg': 
	        imageName = fileName + '.jpg';
		break;
	    case 'image/gif': 
	        imageName = fileName + '.gif';
	    	break;
	    case 'image/png': 
	        imageName = fileName + '.png';
		break;
	    case 'image/bmp': 
	        imageName = fileName + '.bmp';
	    	break;
	    case 'image/tiff': 
	        imageName = fileName + '.tiff';
	    	break;
	    default:
		this.printLog("UnKnown image type: " + imageType);
		return null;
	}
	return imageName;
    },

    readAsciFile: function(file) {
	var data = "";
	var fileInputStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                        .createInstance(Components.interfaces.nsIFileInputStream);
	var scriptableInputStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
                        .createInstance(Components.interfaces.nsIScriptableInputStream);
	try {
	    fileInputStream.init(file, 1, 0, false);
	    scriptableInputStream.init(fileInputStream); 

	    var str = scriptableInputStream.read(-1);
	    while (str.length > 0) {
		data += str;
		str = scriptableInputStream.read(-1);
	    }
	    scriptableInputStream.close();
	    fileInputStream.close();
	} catch (e) {
	    this.printLog("Failed to read file: " + file.path + " - " + e);
	    try { 
		scriptableInputStream.close();
		fileInputStream.close();
	    } catch (e) {}
	    return null;
	}
	return data;
    },

    generateCacheFileName: function(albumOrTrackName, artist, imageSize, bIsTrack) {
        //the max size of file name in mozilla is 100
        var encodedArtist = encodeURIComponent(artist.toLowerCase());
        encodedArtist = encodedArtist.replace(/%/g, '');
        var encodedAlbumOrTrack = encodeURIComponent(albumOrTrackName.toLowerCase());
        encodedAlbumOrTrack = encodedAlbumOrTrack.replace(/%/g, '');

        var maxChars = 80;
        var halfMaxChar = maxChars / 2;

        if (encodedArtist.length + encodedAlbumOrTrack.length > maxChars) {
            if (encodedArtist.length > halfMaxChar &&  encodedAlbumOrTrack.length > halfMaxChar ) {
                // if both artist an album are more then halfMaxChar cahrs
                encodedArtist = this.truncateString(encodedArtist, halfMaxChar);
                encodedAlbumOrTrack = this.truncateString(encodedAlbumOrTrack, halfMaxChar);
            } else if (encodedArtist.length > halfMaxChar ) {
                // if just artist is more then halfMaxChar cahrs
                encodedArtist = this.truncateString(encodedArtist, maxChars - encodedAlbumOrTrack.length);
            } else if (encodedAlbumOrTrack.length > halfMaxChar) {
                // if just album is more then halfMaxChar cahrs
                encodedAlbumOrTrack = this.truncateString(encodedAlbumOrTrack, maxChars - encodedArtist.length);
            }
        }

    	var fileName = bIsTrack ? encodedArtist + '_^TR^_' + encodedAlbumOrTrack + "_" + imageSize.toLowerCase() : 
                                  encodedArtist + '_^_' + encodedAlbumOrTrack + "_" + imageSize.toLowerCase();

    	fileName = fileName.replace(/\*/g, '_');
	    return fileName;
    },
    
    truncateString: function (str, length) {
        // taking a long prefix and a shorter suffix to ensure uniqueness
        var res = str.substr(0, length-5) + str.substr(str.length-4);
        return res;
    },    

    isValidateCacheVersion: function (itemVersion) {
	    return true;
    },

    printLog : function(msg) {
        dump(msg + '\n');
    },
}
