// Copyright 2006 FoxyTunes
// All Rights Reserved

function FoxyTunesFeedMenuPopupUI(menupopup, feedURL) {
    this.menupopup = menupopup;
    this.feedURL = feedURL;

    this.startFetch();
}

// static 
FoxyTunesFeedMenuPopupUI.openFeedItem = function(event) {
    var menupopup = event.currentTarget;
    var target = event.target;
    var url    = target.value;

    if (target.isPodcast) {
        if (!foxytunesStreamIsSupportedInCurrentPlayer()) {
            foxytunesAlertStreamNotSupported();
            return;
        } 
        foxytunesInitPlayerObjectIfNeeded();
        foxytunesShowPleaseWait(true, foxytunesGetLocaleString('pleaseWaitWhileTalkingToPlayer'));
        setTimeout(function() { foxytunesDoAndHidePleaseWait(function() { gFoxyTunesPlayerObj.PlayURIs(url); }) }, 100);
    } else if (target.isSubscribe) {
        url = url.replace(/^http/, 'itpc');
        foxytunesInitPlayerObjectIfNeeded();
        foxytunesShowPleaseWait(true, foxytunesGetLocaleString('pleaseWaitWhileTalkingToPlayer'));
        setTimeout(function() { foxytunesDoAndHidePleaseWait(function() { gFoxyTunesPlayerObj.PlayURIs(url); }) }, 100);
    } else {
        var urls = url.split('\t');
        if (urls.length == 1) {
            foxytunesGoToURL(url);
        } else {
            foxytunesOpenInTabs(urls);
        }
    }

    event.stopPropagation();
}

FoxyTunesFeedMenuPopupUI.prototype = {
    startFetch : function() {
        var me = this;
        this.setStatus(foxytunesGetLocaleString("fetching"));

        var acceptedMimeTypes = ['text/xml', 
                                 'text/html', 
                                 'application/xml', 
                                 'application/rss+xml', 
                                 'application/rdf+xml', 
                                 'application/atom+xml', 
                                 'text/plain'];
        var request = new XMLHttpRequest();            
        request.open('get', this.feedURL, true);
        request.overrideMimeType('text/xml');
        request.setRequestHeader('Accept', acceptedMimeTypes.join(','));
        request.onreadystatechange = function() {
                if (request.readyState == 2) {
                    // headers ready
                    var contentType = request.getResponseHeader("Content-Type");                    
                    contentType = contentType.toLowerCase();
                    contentType = contentType.split(';')[0];
                    if (acceptedMimeTypes.indexOf(contentType) == -1) {
                        // we are not expecting this mime type - abort
                        request.abort();
                        me.onError();
                        return;
                    }
                }

                if (request.readyState == 4) {
                    if (request.status == 200) {
                        var xml = request.responseXML;
                        var feed = new FTFeed(me.feedURL);
                        if (xml && feed.parse(xml)) {
                            me.populate(feed);
                        } else {
                            me.onError();
                        }
                    } else {
                        me.onError();
                    }
                }
       }
       request.send(null);      
    },

    onError : function() {
        this.setStatus(foxytunesGetLocaleString("errorFetching"));
    },

    setStatus : function(text) {
        this.clearItems();
        var menuitem = document.createElement('menuitem');
        menuitem.setAttribute('label', text);
        menuitem.setAttribute('disabled', 'true');
        this.menupopup.appendChild(menuitem);
    },

    populate : function(feed) {
        //dump(feed);
        this.clearItems();
        this.menupopup.isPodcast = feed.isPodcast;
        var playAllMenuItem = null;
        var openAllMenuItem = null;
        if (feed.isPodcast) {
            var headerMenuItem = document.createElement('menuitem');
            headerMenuItem.setAttribute('label', foxytunesGetLocaleString('podcastWithTitle', [feed.title]));
            headerMenuItem.setAttribute('tooltiptext', foxytunesGetLocaleString('podcastWithTitle', [feed.title]));
            headerMenuItem.setAttribute('class', 'menuitem-iconic foxytunes-podcast-item foxytunes-icon-header-menuitem');
            headerMenuItem.setAttribute('disabled', 'true');
            this.menupopup.appendChild(headerMenuItem);

          	var player = foxytunesTryGetFoxyTunesPlayerFromContractID(gFoxyTunesCurrentPlayerClass);

            playAllMenuItem = document.createElement('menuitem');
            playAllMenuItem.setAttribute('label', foxytunesGetLocaleString('streamAll'));
            playAllMenuItem.setAttribute('tooltiptext', foxytunesGetLocaleString('streamAll'));
            playAllMenuItem.setAttribute('class', 'menuitem-iconic player-item'); 
          	playAllMenuItem.setAttribute('playername', player.toLowerCase());
            playAllMenuItem.isPodcast = true;
            this.menupopup.appendChild(playAllMenuItem);

         
            var bSupportsPodcast = gFTPlayerConfig.getSupportsSubscribeToPodcast(player.toLowerCase());
            if (bSupportsPodcast) {
                var subscribeMenuItem = document.createElement('menuitem');
                subscribeMenuItem.setAttribute('label', foxytunesGetLocaleString('subscribeToPodcast'));
                subscribeMenuItem.setAttribute('tooltiptext', foxytunesGetLocaleString('subscribeToPodcast'));
                subscribeMenuItem.setAttribute('class', 'menuitem-iconic player-item'); 
            	subscribeMenuItem.setAttribute('playername', player.toLowerCase());
                subscribeMenuItem.isSubscribe = true;
                subscribeMenuItem.value = feed.feedURL;
                //subscribeMenuItem.setAttribute('class', 'menuitem-iconic foxytunes-podcast-item');
                this.menupopup.appendChild(subscribeMenuItem);
            }
        } else {
            var headerMenuItem = document.createElement('menuitem');
            headerMenuItem.setAttribute('label', foxytunesGetLocaleString('feedWithTitle', [feed.title]));
            headerMenuItem.setAttribute('tooltiptext', foxytunesGetLocaleString('feedWithTitle', [feed.title]));
            headerMenuItem.setAttribute('class', 'menuitem-iconic foxytunes-icon-header-menuitem');
            headerMenuItem.setAttribute('disabled', 'true');
            this.menupopup.appendChild(headerMenuItem);

            openAllMenuItem = document.createElement('menuitem');
            openAllMenuItem.setAttribute('label', foxytunesGetLocaleString('openInTabs'));
            openAllMenuItem.setAttribute('tooltiptext', foxytunesGetLocaleString('openInTabs'));
            openAllMenuItem.setAttribute('class', 'menuitem-iconic');
            this.menupopup.appendChild(openAllMenuItem);          
        }

        var menuseparator = document.createElement('menuseparator');
        this.menupopup.appendChild(menuseparator);

        var allEnclosures = [];
        var allLinks = [];
        var items = feed.items;
        for (var i=0; i < items.length; i++) {
            var menuitem = document.createElement('menuitem');
            menuitem.setAttribute('label', items[i].title);
            menuitem.value = items[i].enclosure ?  items[i].enclosure : items[i].location;
            menuitem.setAttribute('tooltiptext', menuitem.value);
            if (items[i].enclosure) {
                menuitem.setAttribute('class', 'menuitem-iconic podcast-item');
                menuitem.isPodcast = true;
                allEnclosures.push(menuitem.value);
            } else {
                menuitem.setAttribute('class', 'menuitem-iconic feed-item');
                allLinks.push(menuitem.value);
            }
            this.menupopup.appendChild(menuitem);
        }

        if (playAllMenuItem) {
            playAllMenuItem.value = allEnclosures.join('\t');
        } else {
            openAllMenuItem.value = allLinks.join('\t');
        }
    },

    clearItems : function() {
        foxytunesClearMenupopup(this.menupopup);
    }
}

