/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FoxyTunes Mozilla Extension and Engine.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function foxytunesInstallDragDropObserversForElementById(elementId, bInstall) {
   var elem = document.getElementById(elementId);
   if (elem == null) {
	dump("\nAdding observer, cannot find element: " + elementId + "\n");
	return;
   }

   if (bInstall) {
  	elem.addEventListener("dragover", foxytunesOnTargetDragOver, false);
	elem.addEventListener("dragexit", foxytunesOnTargetDragExit, false);
	elem.addEventListener("dragdrop", foxytunesOnTargetDragDrop, false);
   } else {
  	elem.removeEventListener("dragover", foxytunesOnTargetDragOver, false);
	elem.removeEventListener("dragexit", foxytunesOnTargetDragExit, false);
	elem.removeEventListener("dragdrop", foxytunesOnTargetDragDrop, false);
   }
}

function foxytunesInstallUninstallDragDropObservers(bInstall) {
  foxytunesInstallDragDropObserversForElementById('status-bar', bInstall);

  // all the toolboxes:
  var toolboxes = document.getElementsByTagName('toolbox');
  for (var i = 0; i < toolboxes.length; i++) {
 	foxytunesInstallDragDropObserversForElementById(toolboxes[i].id, bInstall);
  }

  // foxytunesInstallDragDropObserversForElementById('navigator-toolbox', bInstall);
}

function foxytunesInstallDragDropObservers() {
   foxytunesInstallUninstallDragDropObservers(true);
}

function foxytunesUnInstallDragDropObservers() {
   foxytunesInstallUninstallDragDropObservers(false);
}


function foxytunesHasDropClass(className) {
  var classNames = className.split(" ");
  for (var i = 0; i < classNames.length; i++) {
	if (classNames[i].indexOf('foxytunes-drop-target-') != -1) {
		return true;
	}
  }
  return false;
}

function foxytunesRemoveDropClass(className) {
  var classNames = className.split(" ");

  if (classNames.length < 1) {
  	return className;
  }

  // see if the last class is foxytunes drop class:
  if (classNames[classNames.length - 1].indexOf('foxytunes-drop-target-') != -1) {
	classNames.length--; // truncate the array
        if (classNames.length > 0) {
		className = classNames.join(" ");
	} else {
		className = "";
	}
  } 
  return className;
}

function foxytunesElementIsToolbarOrStatusbar(elem) {
  return (elem.localName == "toolbar") || (elem.localName == "statusbar") || (elem.localName == "menubar");
}

function foxytunesSetDropTargetMarker(node, bSet) {
  var target = node;
  var side = 'left';
  if (foxytunesElementIsToolbarOrStatusbar(node)) {
    target = node.lastChild;
    side = "right";
  }
  
  if (target == null) {
  	return;
  }

  if (bSet) {
	if (!foxytunesHasDropClass(target.className)) {
		var dropClassName = 'foxytunes-drop-target-' + side;
		target.className = target.className + " " + dropClassName;
		//dump("\n new Class " + target.className + "\n");
	}
  } else {
	target.className = foxytunesRemoveDropClass(target.className);
	//dump("\nRestored Class " + target.className + "\n");
  } 
}


function foxytunesOnTargetDragOver(event)
{
  nsDragAndDrop.dragOver(event, foxyTargetObserver);
}

function foxytunesOnTargetDragExit(event)
{
   if (gFoxyTunesCurrentDropTarget != null) {
	   foxytunesSetDropTargetMarker(gFoxyTunesCurrentDropTarget, false);
   }
}

function foxytunesOnTargetDragDrop(event)
{
  nsDragAndDrop.drop(event, foxyTargetObserver);
}

var foxyDragStartObserver =
{
  onDragStart: function (event, transferData, action) {
	foxytunesInstallDragDropObservers();
	transferData.data = new TransferData();
	transferData.data.addDataForFlavour('id/foxytunes-widget', 'foxytunes-statusbar-panel');
  }
}


var foxyTargetObserver =
{
  onDragOver: function (event, flavour, session)
  {
   //dump("over\n");
   var topElement = event.target;
   var target = event.target;
   while (topElement && !foxytunesElementIsToolbarOrStatusbar(topElement)) {
	      target = topElement;
	      topElement = topElement.parentNode;
   } 
   
   var previousDragItem = gFoxyTunesCurrentDropTarget;

   if (foxytunesElementIsToolbarOrStatusbar(target)) {
	 gFoxyTunesCurrentDropTarget = target;
   } else {
      	 var targetWidth = target.boxObject.width;
         var targetX = target.boxObject.x;

         gFoxyTunesCurrentDropTarget = null;
         if (event.clientX > (targetX + (targetWidth / 2))) {
	        gFoxyTunesCurrentDropTarget = target.nextSibling;
//		dump("\ncrossed\n");
        	if (gFoxyTunesCurrentDropTarget == null) {
			  // last element in its parent, set target to parent
		          gFoxyTunesCurrentDropTarget = topElement;
		}
      	 } else {
        	gFoxyTunesCurrentDropTarget = target;
    	 }    
   }

 //  dump("\nprev: " + previousDragItem.id + ", next: " + gFoxyTunesCurrentDropTarget.id + "\n");
   if (previousDragItem && (gFoxyTunesCurrentDropTarget != previousDragItem)) {
	foxytunesSetDropTargetMarker(previousDragItem, false);
   }
 
   if (gFoxyTunesCurrentDropTarget.id.indexOf('foxytunes') == -1) { 
	   foxytunesSetDropTargetMarker(gFoxyTunesCurrentDropTarget, true);
	   session.canDrop = true;
   } else {
   	   // cannot drop on myself: 
	   foxytunesSetDropTargetMarker(gFoxyTunesCurrentDropTarget, false);
	   gFoxyTunesCurrentDropTarget = null;
	   session.canDrop = false;
   }
  },
 
  onDragExit: function (event, session) 
  {
	//dump("exit\n");
  },

  onDrop: function (event, dropData, session)
  {
	//dump("drop\n");
	foxytunesUnInstallDragDropObservers();
        if (gFoxyTunesCurrentDropTarget == null) {
		return; 
	}
        foxytunesSetDropTargetMarker(gFoxyTunesCurrentDropTarget, false);
    
	var draggedItemId = dropData.data;
	// sanity, should never happen:
	if (gFoxyTunesCurrentDropTarget.id == draggedItemId) {
	      	return;
	}

	var topElement = event.target;
        while (topElement && !foxytunesElementIsToolbarOrStatusbar(topElement)) {
	      topElement = topElement.parentNode;
        }

	// save the new settings:
	gFoxyTunesParentElementID = topElement.id;
	gFoxyTunesInsertBeforeElementId = gFoxyTunesCurrentDropTarget.id;

	// for the case when the "insert before" element is a dynamic one, remember the 
	// "insert after" element
	if (gFoxyTunesCurrentDropTarget.previousSibling) {	
	 	gFoxyTunesInsertAfterElementId = gFoxyTunesCurrentDropTarget.previousSibling.id;
	}

	foxytunesWritePreferences();
	
	foxytunesSetFoxyTunesPosition();

	dump("\nInserted to: " + topElement.id + ", before " + gFoxyTunesCurrentDropTarget.id + "\n");
	gFoxyTunesCurrentDropTarget = null;
  },
  
  
  getSupportedFlavours: function ()
  {
	var flavours = new FlavourSet();
	flavours.appendFlavour("id/foxytunes-widget");
	return flavours;
  }
}


function foxytunesRenameTagName(elem, newTagName) {
	var newElem = document.createElement(newTagName);

	// copy all the attributes of the element
	for (var i=0; i < elem.attributes.length; i++) {
		newElem.setAttribute(elem.attributes[i].nodeName, elem.attributes[i].nodeValue);
	}

	// move all the children
	var children = elem.childNodes;
	for (var i=children.length-1; i >=0 ; i--) {
        var currentNode = children[i];
		elem.removeChild(currentNode);
		newElem.insertBefore(currentNode, newElem.firstChild);
	} 

  	return newElem;	
}

function foxytunesSetFoxyTunesPosition() {
    	if ((gFoxyTunesParentElementID == '') ||
		(gFoxyTunesInsertBeforeElementId == '')) {
			// nothing's set, return
			return;
	}

	var foxytunesWidget = document.getElementById('foxytunes-statusbar-panel');

	var parentElement = document.getElementById(gFoxyTunesParentElementID);
	if (parentElement == null) {
		return;
	}
	dump("\nParent element: " + gFoxyTunesParentElementID + "\n");

	var insertBeforeElement = document.getElementById(gFoxyTunesInsertBeforeElementId);
	var insertAfterElement = document.getElementById(gFoxyTunesInsertAfterElementId);

	if ((insertBeforeElement == null) && (insertAfterElement == null)) {
		return;
	}

	dump("\nInsert before element: " + gFoxyTunesInsertBeforeElementId + "\n");
	dump("\nInsert after element: " + gFoxyTunesInsertAfterElementId + "\n");

	var oldParentNode = foxytunesWidget.parentNode;

	foxytunesWidget.parentNode.removeChild(foxytunesWidget);

	try {
		// make FoxyTunes toolbaritem and not statubarpanel if needed:
		if ((parentElement.localName == 'toolbar') && (foxytunesWidget.localName == 'statusbarpanel')) {
			foxytunesWidget = foxytunesRenameTagName(foxytunesWidget, 'toolbaritem');
		}

		// make FoxyTunes statusbarpanel and not toolbaritem if needed:
		if ((parentElement.localName == 'statusbar') && (foxytunesWidget.localName == 'toolbaritem')) {
			foxytunesWidget = foxytunesRenameTagName(foxytunesWidget, 'statusbarpanel');
		}

		// convention, if the parent equals insertbefore, insert as last
		if (parentElement != insertBeforeElement) {
			if (insertBeforeElement) {
				parentElement.insertBefore(foxytunesWidget, insertBeforeElement);
			} else {
				dump('insert before failed try inserting after ' + gFoxyTunesInsertAfterElementId + '\n');
				if (insertAfterElement.nextSibling) {
					parentElement.insertBefore(foxytunesWidget, insertAfterElement.nextSibling);
				} else {
					parentElement.appendChild(foxytunesWidget);
				}
			}
		} else {
			parentElement.appendChild(foxytunesWidget);
		}

	} catch (err) {
		dump("\nCouldn't reposition FoxyTunes: " + err + "\n");
		oldParentNode.appendChild(foxytunesWidget);
	}

    // need to re-setup the volume slider listeners:
	foxytunesRegisterVolumeSliderEvents();

    // adjust popup positions (top/bottom):
    //foxytunesAdjustPopupPositions();
}

// the current drop target:
var gFoxyTunesCurrentDropTarget = null;
