// Copyright 2006 FoxyTunes
// All Rights Reserved

/********* FoxyTunesPandoraController *********/

function FoxyTunesPandoraController() {
    this._windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Components.interfaces.nsIWindowMediator);

    this._volumeController = null;

    const kVolumeControllerCID = "@foxytunes.org/FoxyTunesEngine/FoxyTunesVolumeController;1";
    if (typeof Components.classes[kVolumeControllerCID] != 'undefined') {
        this._volumeController = Components.classes[kVolumeControllerCID]
                    .getService(Components.interfaces.IFoxyTunesVolumeController);        
    }

    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils/trackDataBuilder.js");
    }

    this._lastWindowAndTab = null;
    this._loadEventListener = null;
}

FoxyTunesPandoraController.prototype = {
    Play : function() {
        return this.ShowPlayer();
    },

    Pause : function() {
        return this.ShowPlayer();
    },

    ForcePause : function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    Stop : function() {
        return this.ShowPlayer();
    },

    Next : function() {
        return this.ShowPlayer();
    },

    Previous : function() {
        return this.ShowPlayer();
    },


    ToggleMute : function() {
        if (this._volumeController) {
            return this._volumeController.ToggleMute();
        } else {
            return this.ShowPlayer();
        }
    },

    SetVolume : function(value) {
        if (this._volumeController) {
            return this._volumeController.SetVolume(value);
        } else {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
    },

    GetVolume : function() {
        if (this._volumeController) {
            return this._volumeController.GetVolume();
        } else {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
    },

    IncreaseVolume : function() {
        if (this._volumeController) {
            var volume = this._volumeController.GetVolume();
            var newVolume = volume + 10;
            if (newVolume > 100) {
                newVolume = 100;
            }
            this._volumeController.SetVolume(newVolume);
        } else {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
    },

    DecreaseVolume : function() {
        if (this._volumeController) {
            var volume = this._volumeController.GetVolume();
            var newVolume = volume - 10;
            if (newVolume < 0) {
                newVolume = 0;
            }
            this._volumeController.SetVolume(newVolume);
        } else {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
    },

    GenerateTrackDataXml : function(title) {
        var dataBuilder = new TrackDataBuilder();
        dataBuilder.addTitle(title);
        dataBuilder.addDisplayName(title);
        dataBuilder.addIsVideo("no");
        return dataBuilder.getData();
    },

    GetCurrentTrackTitle : function() {
        var bAttachToTab = true;
        var listener = null;
        var pandoraWindow = this.findPandoraWindow();
        if (pandoraWindow) {
            var doc = pandoraWindow.content.document;
            listener = this.attachEventListenerIfNeeded(doc);                  
            var title = this.getTrackTitleFromWindow(doc, listener);
            if (title) {
                return this.GenerateTrackDataXml(title);
            } else {
                bAttachToTab = false;
            }
        }

        var foundTabInfo = this.findPandoraTab();
        if (foundTabInfo) {
            if (foundTabInfo.source == 'pandora') {
                var doc = foundTabInfo.doc;
                if (bAttachToTab) {
                    listener = this.attachEventListenerIfNeeded(doc);
                }
                return this.GenerateTrackDataXml(this.getTrackTitleFromWindow(doc, listener));
            } else if (foundTabInfo.source == 'pandorafm') {
                var title = foundTabInfo.title;
                if (title.match(/(.* - .*) - (.*) - PandoraFM/i)) {
                    return this.GenerateTrackDataXml(RegExp.$1);
                } else if (title.match(/(.*) - PandoraFM/i)) {
                    return this.GenerateTrackDataXml(RegExp.$1);
                }
            }
        }

        return null;
    },

    GetCurrentTrackLength : function() {
       //trashes the console: throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
       return 0;
    },

    GetCurrentTrackPosition : function() {
       //trashes the console: throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
       return 0;
    },

    ShowPlayer : function() {

        var pandoraWindow = this.findPandoraWindow();
        if (pandoraWindow) {
            pandoraWindow.focus();
            return true;
        } 

        var foundTabInfo = this.findPandoraTab();
        if (foundTabInfo) {
            var window  = foundTabInfo.window;
            var browser = foundTabInfo.browser;
            var tab     = foundTabInfo.tab;
            var tuner     = foundTabInfo.tuner;
            
            var lastWindowAndTab = this.getCurrentWindowAndTab();
            if (lastWindowAndTab && !lastWindowAndTab[2].match(/pandora.com/)) {
                // do not remember pandora tabs
                this._lastWindowAndTab = lastWindowAndTab;
            } 

            window.focus();
            browser.selectedTab = tab;
            if (tuner) {
                tuner.scrollIntoView(true);  
                tuner.focus();
            }
            return true;
        }
        
        this.openPandora();
        return false;
    },

    HidePlayer : function() {
        var pandoraWindow = this.findPandoraWindow();
        if (pandoraWindow) {
            pandoraWindow.minimize();
        } else {
            if (this._lastWindowAndTab) {
                var currentWindowAndTab = this.getCurrentWindowAndTab(); 
                if (!currentWindowAndTab[2].match(/pandora.com/) && 
                        !currentWindowAndTab[2].match(/pandorafm.real-ity.com/)) {
                    // do not hide pandora if we ain't in pandora :)
                    return false;
                }

                var lastWindow = this._lastWindowAndTab[0];
                var lastTab = this._lastWindowAndTab[1];

                this._lastWindowAndTab = null;

                try {
                    if (lastWindow) lastWindow.focus();
                    if (lastWindow && lastTab) { 
                        lastWindow.getBrowser().selectedTab = lastTab;
                    }                  
                } catch(e) {}
                return true;
            }
            return false;
        } 
    },

    PlayFile : function() {
        return this.ShowPlayer(); 
    },

    SetOptions  : function(options) {
        return true;
    },

    SetCurrentTrackPosition : function(pos) {
       throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    IsMuted : function() {
        if (this._volumeController) {
            return this._volumeController.IsMuted();
        } else {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
    },


    PlayURIs : function(URIs) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

	// nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.mozIFoxyTunes) ||
			iid.equals(Components.interfaces.nsISupports)) {
               return this;
		}

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
    },

    // private
    getCurrentWindowAndTab : function() {
        var lastWindow = this._windowMediator.getMostRecentWindow('navigator:browser');
        if (!lastWindow) {
            lastWindow = this._windowMediator.getMostRecentWindow('foxytunes:minibrowser');
        }
        if (!lastWindow) return null;

        var lastTabbrowser = lastWindow.getBrowser();
        var lastTab = lastTabbrowser.selectedTab; 
        var browser = lastTabbrowser.getBrowserForTab(lastTab);
        var uri = browser.currentURI.spec;
        return [lastWindow, lastTab, uri];
    },

    findPandoraWindow : function() {
        var windows = this._windowMediator.getEnumerator('navigator:browser');
        while (windows.hasMoreElements()) {
            var wantedWindow = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
            // only search popup windows
            if (!wantedWindow.toolbar.visible &&
                  wantedWindow.document.title.match(/http:\/\/www.pandora.com(.*)Pandora/)) {
                return wantedWindow;
            }
        }
        return null;
    },

    findPandoraTab : function() {
      try {

        var typeList = ['navigator:browser', 'foxytunes:minibrowser'];
        for (var i in typeList) {
            var windows = this._windowMediator.getEnumerator(typeList[i]);
            while (windows.hasMoreElements()) {
                var wantedWindow = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
                // we do not want popup windows
                if (!wantedWindow.toolbar.visible) continue;

                var tabbrowser = wantedWindow.getBrowser();
                var tabs = tabbrowser.mTabContainer.childNodes;
                for (var iTab=0; iTab < tabs.length; iTab++) {
                    var tab = tabs[iTab];
                    var browser = tabbrowser.getBrowserForTab(tab);
                    var uri = browser.currentURI.spec;
                    var bIsPandora = uri.match(/pandora.com/);
                    var bIsPandoraFM = uri.match(/pandorafm.real-ity.com/);
        
                    if (!bIsPandora && !bIsPandoraFM) continue;
            
                    var doc = null;
                    if (bIsPandoraFM) {
                        var contentWindow = browser.contentWindow;
                        var docs = this.getAllDocs(contentWindow);
                        for (var i=0; i < docs.length; i++) {
                            if (docs[i].location.hostname.match(/pandora.com/)) {
                                doc = docs[i];
                                break;
                            }
                        }
                    } else {
                        doc = browser.contentDocument;
                    }

                    if (!doc) continue;

                    var embed = null;
                    if (doc.embeds)
                    {
                        for (var iEmbed = 0; iEmbed < doc.embeds.length; iEmbed++)
                        {
                            var currEmbed = doc.embeds[iEmbed];
                            if (currEmbed.getAttribute('src').match(/pandora.com.*tuner/)) {
                                embed = currEmbed;
                                break;
                            }
                        }
                    }

                    // we need either
                    // 1. pandora player embedded
                    // 2. a pandorafm tab (player can be embedded and can be ditached/minimized)
                    if (!embed && !bIsPandoraFM) continue;

                    var title = bIsPandora ? doc.title : browser.contentDocument.title;

                    var findReply = new Object();
                    findReply.window  = wantedWindow;
                    findReply.browser = tabbrowser;
                    findReply.tab     = tab;
                    findReply.title   = title;
                    findReply.tuner   = embed;
                    findReply.doc     = doc;
                    findReply.source  = bIsPandora ? 'pandora' : 'pandorafm';
                    return findReply;
                } 
            }
        }
     } catch(e) { 
        dump('Error trying to find Pandora tab: ' + e + '\n');
     }

     return null;
    },

    getAllDocs : function(frame) {
        var docs = [];

        if (!frame) return docs;

        if (frame.document) docs.push(frame.document);

        var frames = frame.frames;               
        for (var i = 0; i < frames.length; i++) {
            docs = docs.concat(this.getAllDocs(frames[i]));
        }

        return docs;
    },

    getMostRecentBrowserWindow : function() {
        var win = this._windowMediator.getMostRecentWindow('navigator:browser');
        if (win && win.toolbar.visible) {
            return win;
        }

        // else
        var windows = this._windowMediator.getEnumerator('navigator:browser', true);
        while (windows.hasMoreElements()) {
           win = windows.getNext();
           if (win.toolbar.visible) return win; 
        }

        return null;
    },


    pandoraDocLoaded : function(event) {
        var doc = event.originalTarget;
        var topDoc = doc.defaultView.top.document;
        if (doc != topDoc) {
            // internal frame loaded, ignore
            return;
        }
        this.attachEventListenerIfNeeded(topDoc);


        var browser = event.currentTarget;
        browser.removeEventListener('DOMContentLoaded', this._loadEventListener, false);
        this._loadEventListener = null;

        this.printTrace('Loaded: ' + topDoc.title);
    },

    openPandora : function() {
            // Ignore if in eMusic
            if (this.isInEmusic()) return;

            var me = this;
            const pandoraHome = 'http://www.pandora.com/';
            var win = this.getMostRecentBrowserWindow();
            if (win) {
                var tabbrowser = win.getBrowser();

           		var newTab = tabbrowser.addTab(pandoraHome);
                tabbrowser.selectedTab = newTab;
                
                var browser = tabbrowser.getBrowserForTab(newTab);
                var contentWindow = browser.contentWindow;
                this._loadEventListener = function(event) { me.pandoraDocLoaded(event); };
                browser.addEventListener('DOMContentLoaded', this._loadEventListener, false);
            } else {
                var windowWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                                    .getService(Components.interfaces.nsIWindowWatcher);
                var array = Components.classes["@mozilla.org/supports-array;1"]
                                .createInstance(Components.interfaces.nsISupportsArray);
                var url = Components.classes["@mozilla.org/supports-string;1"]
                                .createInstance(Components.interfaces.nsISupportsString);
                url.data = pandoraHome;

                array.AppendElement(url);
                windowWatcher.openWindow(windowWatcher.activeWindow,
                                 "chrome://browser/content/browser.xul",
                                 "_blank",
                                 "chrome,all,dialog=no",
                                 array);
           }
    },

    attachEventListenerIfNeeded : function(doc) {
        var listener = doc.getElementById('FTEventListener');
        if (listener) return listener;

        var div = doc.createElement('div');
        div.id = 'FTEventListener';
        div.innerHTML = '<script> function ftSetCurrentPandoraSong(song, artist) { var div = document.getElementById("FTEventListener"); div.setAttribute("artist", artist); div.setAttribute("song", song); }; function ftRegisterPandoraEvents() { Pandora.setEventHandler("SongPlayed", function(songData) { ftSetCurrentPandoraSong(songData.songName, songData.artistName); }); Pandora.setEventHandler("SongEnded", function(songData) { ftSetCurrentPandoraSong("",  ""); }); } if (typeof(Pandora) == "undefined") { var script=document.createElement("script"); script.setAttribute("src", "http://www.pandora.com/include/PandoraAPIv2.js"); script.setAttribute("type","text/javascript"); script.onload = ftRegisterPandoraEvents; document.getElementsByTagName("head")[0].appendChild(script)} else { ftRegisterPandoraEvents(); } </script>';
        doc.documentElement.appendChild(div);
        return div;
    },
   
    getTrackTitleFromWindow : function(doc, listener) {
        if (!listener) return '';

        var title = '';
        try {
            var artist = listener.getAttribute('artist');
            var song = listener.getAttribute('song');
            if (artist && song) {
                title =  artist + ' - ' + song;
            }
        } catch(e) {}

        return title;
    },

    isInEmusic : function() {
        try {
            var app = Components.classes["@mozilla.org/xre/app-info;1"]
                   .getService(Components.interfaces.nsIXULAppInfo)
                   .QueryInterface(Components.interfaces.nsIXULRuntime);
           return (app.name.search(/emusic/gi) != -1);
        } catch (e) {};
        return false;
    },

    printTrace : function(message) {
        //dump(message + '\n');
    },
}
