// Copyright 2006 FoxyTunes
// All Rights Reserved

function FoxyTunesRhapsodyController() {
    this._windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Components.interfaces.nsIWindowMediator);

    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils/trackDataBuilder.js");
    }
}

FoxyTunesRhapsodyController.prototype = {
    Play : function() {
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (!rhapsodyWindow) {
            this.ShowPlayer();
            return false;
        }
        return this.executeRhapsodyCommand(rhapsodyWindow, 'rhapPlayerPlay()');
    },

    Pause : function() {
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (!rhapsodyWindow) {
            this.ShowPlayer();
            return false;
        }

        return this.executeRhapsodyCommand(rhapsodyWindow, 'rhapPlayerPauseStop()');
    },

    ForcePause : function() {
        return this.Pause();
    },

    Stop : function() {
        return this.Pause();
    },

    Next : function() {
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (!rhapsodyWindow) {
            this.ShowPlayer();
            return false;
        }

        return this.executeRhapsodyCommand(rhapsodyWindow, 'rhapPlayerNext()');
    },

    Previous : function() {
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (!rhapsodyWindow) {
            this.ShowPlayer();
            return false;
        }

        return this.executeRhapsodyCommand(rhapsodyWindow, 'rhapPlayerPrevious()');
    },


    ToggleMute : function() {
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (!rhapsodyWindow) {
            this.ShowPlayer();
            return false;
        }

        return this.executeRhapsodyCommand(rhapsodyWindow, 'rhapPlayerMute()');
    },

    SetVolume : function(value) {
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (!rhapsodyWindow) {
            return false;
        }

        var volume = Math.round(value/10);
        return this.executeRhapsodyCommand(rhapsodyWindow, 'rhapPlayerSetVolume(' + volume + ')');
    },

    GetVolume : function() {
        var defaultVolume = 50;
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (!rhapsodyWindow) {
            return defaultVolume;
        }
        var doc = rhapsodyWindow.content.document;
        var ftRhapsodyBridge = doc.getElementById('ftRhapsodyBridge');
        if (!ftRhapsodyBridge) {
            this.attachFtRhapsodyBridge(doc);
            ftRhapsodyBridge = doc.getElementById('ftRhapsodyBridge');
        }
        if (!ftRhapsodyBridge) {
            return defaultVolume;
        }
        var ftGetVolume = doc.getElementById('ftGetVolume');
        if (!ftGetVolume) {
            return defaultVolume;
        }
        ftGetVolume.click(); 
        var volume = ftRhapsodyBridge.getAttribute('volume');
        return volume;
    },

    IncreaseVolume : function() {
        var volume = this.GetVolume();
        if (volume >= 100) {
            return false;
        }
        volume = parseInt(volume) + 10;
        this.SetVolume(volume);
        return true;
    },

    DecreaseVolume : function() {
        var volume = this.GetVolume();
        if (volume <= 0) {
            return false;
        }
        volume = parseInt(volume) - 10;
        this.SetVolume(volume);
        return true;
    },

    GetCurrentTrackTitle : function() {
        var dataBuilder = new TrackDataBuilder();

        var rhapsodyWindow = this.findRhapsodyWindow();
        if (!rhapsodyWindow) {
            return dataBuilder.getData();
        }

        track = this.getRhapsodyAAT(rhapsodyWindow, 'track');
        dataBuilder.addTitle(track);

        artist = this.getRhapsodyAAT(rhapsodyWindow, 'artist');
        dataBuilder.addArtist(artist);

        album = this.getRhapsodyAAT(rhapsodyWindow, 'album');
        dataBuilder.addAlbum(album);

        var displayName = '';
        if (artist) { 
            displayName = artist;
            if (track) {
                displayName += ' - ' + track;
            }
        } else if (track) {
            displayName = track;
        }
        dataBuilder.addDisplayName(displayName);

        var albumCoverURI = this.getRhapsodyAlbumCoverURI(rhapsodyWindow);
        dataBuilder.addAlbumCoverURI(albumCoverURI);

        dataBuilder.addIsVideo("no");

        return dataBuilder.getData();
    },

    GetCurrentTrackLength : function() {
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (!rhapsodyWindow) {
            return false;
        }

        var trackTimeArr = this.getTrackTimeArr(rhapsodyWindow);
        var trackLength = this.timeStrToSeconds(trackTimeArr[1]);
        return trackLength;
    },

    GetCurrentTrackPosition : function() {
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (!rhapsodyWindow) {
            return false;
        }

        var trackTimeArr = this.getTrackTimeArr(rhapsodyWindow);
        var trackPosition = this.timeStrToSeconds(trackTimeArr[0]);
        return trackPosition; 
    },

    ShowPlayer : function() {
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (rhapsodyWindow) {
            rhapsodyWindow.focus();
            return true;
        }

        this.openRhapsody();
        return false;
    },

    HidePlayer : function() {
        var rhapsodyWindow = this.findRhapsodyWindow();
        if (rhapsodyWindow) {
            rhapsodyWindow.minimize();
            return true;
        }
        return false;
    },

    PlayFile : function() {
        return this.ShowPlayer();
    },

    SetOptions  : function(options) {
        return true;
    },

    SetCurrentTrackPosition : function(pos) {
       throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    IsMuted : function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },


    PlayURIs : function(URIs) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

	// nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.mozIFoxyTunes) ||
			iid.equals(Components.interfaces.nsISupports)) {
               return this;
		}

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
    },

      // private
    findRhapsodyWindow : function() {
        var windows = this._windowMediator.getEnumerator('navigator:browser');
        while (windows.hasMoreElements()) {
            var wantedWindow = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
            // only search popup windows
            if (!wantedWindow.toolbar.visible &&
                  (wantedWindow.document.title.match(/Rhapsody Player/) ||
                   wantedWindow.document.title.match(/Comcast Rhapsody/))) {
                return wantedWindow;
            }
        }
        return null;    
    },

    getMostRecentBrowserWindow : function() {
        var win = this._windowMediator.getMostRecentWindow('navigator:browser');
        if (win && win.toolbar.visible) {
            return win;
        }

        // else
        var windows = this._windowMediator.getEnumerator('navigator:browser', true);
        while (windows.hasMoreElements()) {
           win = windows.getNext();
           if (win.toolbar.visible) return win; 
        }

        return null;
    },

    openRhapsody : function() {
        // Ignore if in eMusic
        if (this.isInEmusic()) return;

        const rhapsodyHome = "http://www.qksrv.net/click-2115496-10436574?sid=foxy_client&url=http%3A%2F%2Fwww.rhapsody.com%3Fpcode%3Dcj%26cpath%3Daff"
        var win = this.getMostRecentBrowserWindow();
            if (win) {
                var tabbrowser = win.getBrowser();

           		var newTab = tabbrowser.addTab(rhapsodyHome);
                tabbrowser.selectedTab = newTab;
            } else {
                var windowWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                                    .getService(Components.interfaces.nsIWindowWatcher);
                var array = Components.classes["@mozilla.org/supports-array;1"]
                                .createInstance(Components.interfaces.nsISupportsArray);
                var url = Components.classes["@mozilla.org/supports-string;1"]
                                .createInstance(Components.interfaces.nsISupportsString);
                url.data = rhapsodyHome;

                array.AppendElement(url);
                windowWatcher.openWindow(windowWatcher.activeWindow,
                                 "chrome://browser/content/browser.xul",
                                 "_blank",
                                 "chrome,all,dialog=no",
                                 array);
           }
    },

    attachFtRhapsodyBridge : function(doc) {
        var div = doc.createElement('div');
        div.id = 'ftRhapsodyBridge';
        div.style.display = 'none';
        div.innerHTML = '<script> function ftSetVolumeAttr(volume) { var div = document.getElementById("ftRhapsodyBridge"); div.setAttribute("volume", volume);} function ftGetVariable(varName) { var div = document.getElementById("ftRhapsodyBridge"); try {div.setAttribute(varName, eval(varName));} catch (e) {};} </script><button id="ftExecCommand"></button><button id="ftGetVolume" onclick="ftSetVolumeAttr(RhapsodyEvents.getVolume())"></button>';
        doc.documentElement.appendChild(div);
    },

    executeRhapsodyCommand : function(rhapsodyWindow, command) {
        var doc = rhapsodyWindow.content.document;
        var ftRhapsodyBridge = doc.getElementById('ftRhapsodyBridge');
        if (!ftRhapsodyBridge) {
            this.attachFtRhapsodyBridge(doc);
            var ftRhapsodyBridge = doc.getElementById('ftRhapsodyBridge');
            if (!ftRhapsodyBridge) return false;
        }

        var ftExecCommand = doc.getElementById('ftExecCommand');
        if (!ftExecCommand) {
            return false;
        }
        ftExecCommand.setAttribute("onclick", command);
        ftExecCommand.click(); 
        return true;
    },

    getTrackTimeArr : function(rhapsodyWindow) {
        var doc = rhapsodyWindow.content.document;
        var trackLengthElem = doc.getElementById('RhapPlayerPosition');
        if (!trackLengthElem) {return new Array("00:00", "00:00");}
        var val = trackLengthElem.firstChild.nodeValue;
        val = val.replace(/^\s*/,'');
        val = val.replace(/\s*$/,'');
        return val.split('/');
    },

    timeStrToSeconds : function(str) {
        if (!str) return 0;
        var timeArr = str.split(':');
        if (timeArr.length == 1) return str * 1;
        if (timeArr.length == 2) return timeArr[0]*60 + timeArr[1]*1;
        if (timeArr.length == 3) return timeArr[0]*60*60 + timeArr[1]*60 + timeArr[2]*1;
    },

    getRhapsodyAAT : function(rhapsodyWindow, type) {
        var id; 
        switch (type) {
            case 'artist':
                id = 'RhapPlayerMetadataArtist';
                break;
            case 'track':
                id = 'RhapPlayerMetadataTrack';
                break;
            case 'album':
                var usingAlbumArt2 = this.isUsingAlbumArt2(rhapsodyWindow);
                id = usingAlbumArt2 ? 'RhapPlayerAlbumArt2' : 'RhapPlayerAlbumArt1';
                break;
            default:
                return '';
        }

        var res = '';
        var doc =  rhapsodyWindow.content.document;
        var divElem = doc.getElementById(id);
        if (!divElem) return res;
        var linkElem = divElem.getElementsByTagName('a')[0];
        if (!linkElem) return res;
        if (type == 'album') {
            res = linkElem.getAttribute('title'); 
            res = res.replace(/^ *Album: */,'');
        } else {
            res = linkElem.firstChild.nodeValue;
        }
        return res;
    },

    getRhapsodyAlbumCoverURI : function(rhapsodyWindow) {
        var doc =  rhapsodyWindow.content.document;
        var usingAlbumArt2 = this.isUsingAlbumArt2(rhapsodyWindow);
        var albumArtId = usingAlbumArt2 ? 'RhapPlayerAlbumArtImg2' : 'RhapPlayerAlbumArtImg1';
        var albumArtImg = doc.getElementById(albumArtId);
        if (!albumArtImg) {
            return '';
        } else {
            return albumArtImg.getAttribute('src');
        }
    },

    isUsingAlbumArt2 : function(rhapsodyWindow) {
        this.executeRhapsodyCommand(rhapsodyWindow, 'ftGetVariable("usingAlbumArt2")');
        var doc =  rhapsodyWindow.content.document;
        var ftRhapsodyBridge = doc.getElementById('ftRhapsodyBridge');
        if (!ftRhapsodyBridge) {
            this.attachFtRhapsodyBridge(doc);
            ftRhapsodyBridge = doc.getElementById('ftRhapsodyBridge');
        }
        if (!ftRhapsodyBridge) {
            return false;
        }
        var usingAlbumArt2 = ftRhapsodyBridge.getAttribute('usingAlbumArt2');
        // I'm returning !usingAlbumArt2 because Rhapsody does usingAlbumArt2 = !usingAlbumArt2 after showing the image
        // so we need to reverse it
        return (usingAlbumArt2 == 'true') ? false : true;
    },

    isInEmusic : function() {
        try {
            var app = Components.classes["@mozilla.org/xre/app-info;1"]
                   .getService(Components.interfaces.nsIXULAppInfo)
                   .QueryInterface(Components.interfaces.nsIXULRuntime);
           return (app.name.search(/emusic/gi) != -1);
        } catch (e) {};
        return false;
    },

    printTrace : function(message) {
       dump(message + '\n');
    }

}
