// Copyright 2006 FoxyTunes
// All Rights Reserved

function FoxyTunesYahooMediaPlayerController() {
    // Observing dom windows open/close
    this._observerService = Components.classes["@mozilla.org/observer-service;1"]
                            .getService(Components.interfaces.nsIObserverService);
    this._observerService.addObserver(this, "domwindowopened", false);
    this._observerService.addObserver(this, "domwindowclosed", false);
    this._observerService.addObserver(this, "quit-application", false);

    var me = this;
    this._domContentLoadedCallback =  function(event) { me.onDOMContentLoad(event); }
    
    this._windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Components.interfaces.nsIWindowMediator);

    this._timerService = Components.classes["@mozilla.org/timer;1"].
						createInstance(Components.interfaces.nsITimer); 

    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils/trackDataBuilder.js");
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils/chromeToContentBridge.js");
    }

    this.chromeToContentBridge = new FoxyTunesChromeToContentBridge();

    this._lastWindowAndTab = null;
    this._currentYMP = null;

    this._lastVol = null;

    this._YMP_STATE_PLAYING = 2;

    this._YMPList = null;
    this.initYMPList();

    this.wrappedJSObject = this;
}

FoxyTunesYahooMediaPlayerController.prototype = {
    Play : function() {
        var YMP = this.findYMP();
        if (!YMP) {
            this.openYMP(); 
            return false;
        }

        this.chromeToContentBridge.fireCommand(YMP.doc, "YAHOO.MediaPlayer.play()");

        return true;
    },

    Pause : function() {
        var YMP = this.findYMP();
        if (!YMP) return false;

        this.chromeToContentBridge.fireCommand(YMP.doc, "YAHOO.MediaPlayer.pause()");
        
        return true;
    },

    ForcePause : function() {
        return this.Pause();
    },

    Stop : function() {
        this.printTrace("Got: " + this._YMPList.length + "\n");

        var YMP = this.findYMP();
        if (!YMP) return false;

        this.chromeToContentBridge.fireCommand(YMP.doc, "YAHOO.MediaPlayer.stop()");
    },

    Next : function() {
        var YMP = this.findYMP();
        if (!YMP) return false;

        this.chromeToContentBridge.fireCommand(YMP.doc, "YAHOO.MediaPlayer.next()");
        
        return true;
    },

    Previous : function() {
        var YMP = this.findYMP();
        if (!YMP) return false;

        this.chromeToContentBridge.fireCommand(YMP.doc, "YAHOO.MediaPlayer.previous()");
        
        return true;
    },

    ToggleMute : function() {
        if (!this._lastVol) {
            this._lastVol = this.GetVolume();
            this.SetVolume(0);
        } else {
            this.SetVolume(this._lastVol);
            this._lastVol = null;
        }
        return true;
    },

    SetVolume : function(value) {
        var outer = this;

        var setVolumeOnTimer = {
            notify: function (aTimer) {
                if (aTimer != outer._timerService) return;
                var YMP = outer.findYMP();
                if (!YMP) return;

                var vol = value / 100;
                outer.chromeToContentBridge.fireCommand(YMP.doc, "YAHOO.MediaPlayer.setVolume(" + vol + ")");
        }
        };

        this._timerService.cancel();
        this._timerService.initWithCallback(setVolumeOnTimer, 100, Components.interfaces.nsITimer.TYPE_ONE_SHOT);  

        return true;
    },

    GetVolume : function() {
        var YMP = this.findYMP();
        if (!YMP) return false;

        var vol = this.chromeToContentBridge.fireCommand(YMP.doc, "YAHOO.MediaPlayer.getVolume()"); 
        return Math.round(vol*100);
    },

    IncreaseVolume : function() {
        var volume = this.GetVolume();
        if (volume >= 100) {
            return false;
        }
        volume = parseInt(volume) + 10;
        this.SetVolume(volume);
        return true;
    },

    DecreaseVolume : function() {
        var volume = this.GetVolume();
        if (volume <= 0) {
            return false;
        }
        volume = parseInt(volume) - 10;
        this.SetVolume(volume);
        return true;
    },

    GetCurrentTrackTitle : function() {
        var dataBuilder = new TrackDataBuilder();

        var YMP = this.findYMP();
        if (!YMP) return dataBuilder.getData();

        var getTrackTitle = '';
        getTrackTitle += 'function() {';
        getTrackTitle += 'var trackData = YAHOO.MediaPlayer.getMetaData();';
        getTrackTitle += 'var res = trackData.title;';
        getTrackTitle += 'if (trackData.artist) {';
        getTrackTitle += '  res += "@@" + trackData.artist;';
        getTrackTitle += '}';
        getTrackTitle += 'if (trackData.album) {';
        getTrackTitle += '  res += "@@" + trackData.album;';
        getTrackTitle += '}';
        getTrackTitle += 'return res;';
        getTrackTitle += '}()';

         var getAlbumArt = '';
        getAlbumArt += 'function() {';
        getAlbumArt += 'var trackData = YAHOO.MediaPlayer.getMetaData();';
        getAlbumArt += 'return trackData.albumart;';
        getAlbumArt += '}()';

        var title = this.chromeToContentBridge.fireCommand(YMP.doc, getTrackTitle); 
        var albumArt = this.chromeToContentBridge.fireCommand(YMP.doc, getAlbumArt); 

        // Replacing unicode apostrophe with regular one
        title = title.replace(/\u2019/g, "'");
        titleArr = title.split('@@');
        var track = titleArr[0] ? titleArr[0] : null;
        var artist = titleArr[1] ? titleArr[1] : null;
        var album = titleArr[2] ? titleArr[2] : null;

        if (track) {
            dataBuilder.addTitle(track);
        }
        if (artist) {
            dataBuilder.addArtist(artist);
        }
        if (album) {
            dataBuilder.addAlbum(album);
        }
        if (albumArt) {
            dataBuilder.addAlbumCoverURI(albumArt);
        }
        dataBuilder.addIsVideo("no");

        var displayName = '';
        if (artist) { 
            displayName = artist;
            if (track) {
                displayName += ' - ' + track;
            }
        } else if (track) {
            displayName = track;
        }
        dataBuilder.addDisplayName(displayName);

        return dataBuilder.getData();
    },

    GetCurrentTrackLength : function() {
        var YMP = this.findYMP();
        if (!YMP) return false;

        var trackLength = this.chromeToContentBridge.fireCommand(YMP.doc, "YAHOO.MediaPlayer.getTrackDuration()"); 
        return trackLength;
    },

    GetCurrentTrackPosition : function() {
        var YMP = this.findYMP();
        if (!YMP) return false;

        var trackPosition = this.chromeToContentBridge.fireCommand(YMP.doc, "YAHOO.MediaPlayer.getTrackPosition()"); 
        return trackPosition; 
    },

    ShowPlayer : function() {
        var YMP = this.findYMP();
        if (!YMP) return false;

        var lastWindowAndTab = this.getCurrentWindowAndTab();
        if (lastWindowAndTab) {
            if (lastWindowAndTab[1] != this._currentYMP.tab) {
                this._lastWindowAndTab = lastWindowAndTab;
            } else {
                // User pressed on ShowPlayer while in YMP tab -> open player if closed
                if (!this.getPlayerViewState(YMP.doc)) {
                    this.setPlayerViewState(YMP.doc, true);
                    return true;
                }

                // Open playlist if closed
                if (!this.getQueueViewState(YMP.doc)) {
                    this.setQueueViewState(YMP.doc, true);
                    return true;
                }
            }
        } 

        var window  = YMP.window;
        var browser = YMP.browser;
        var tab     = YMP.tab;

        window.focus();
        browser.selectedTab = tab;

        if (!this.getPlayerViewState(YMP.doc)) {
            this.setPlayerViewState(YMP.doc, true);
            return true;
        }
        
        return true;
    },

    HidePlayer : function() {
        var currentWindowAndTab = this.getCurrentWindowAndTab(); 

        if (!this._currentYMP) {
            this._currentYMP = this.findYMP();
            if (!this._currentYMP) return;
        }

        // Don't hide if we're not on the YMP tab
        if (currentWindowAndTab[1] != this._currentYMP.tab) {
            return false;
        }
        
        if (!this._lastWindowAndTab) {
            if (this.getPlayerViewState(this._currentYMP.doc)) {
                this.setPlayerViewState(this._currentYMP.doc, false);
            }
             
            return false;
        }

        var lastWindow = this._lastWindowAndTab[0];
        var lastTab = this._lastWindowAndTab[1];

        this._lastWindowAndTab = null;

        try {
            if (lastWindow) lastWindow.focus();
            if (lastWindow && lastTab) { 
                lastWindow.getBrowser().selectedTab = lastTab;
            }                  
        } catch(e) {}

        return true;
    },

    PlayFile : function() {
        return this.ShowPlayer();
    },

    SetOptions  : function(options) {
        return true;
    },

    SetCurrentTrackPosition : function(pos) {
       throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    IsMuted : function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },


    PlayURIs : function(URIs) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

	// nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.mozIFoxyTunes) ||
			iid.equals(Components.interfaces.nsISupports)) {
               return this;
		}

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
    },

    // private

    // nsIObserver
    observe: function(subject, topic, data) {
        switch (topic) {
            case "quit-application" :
                this._observerService.removeObserver(this, "quit-application");
                this._observerService.removeObserver(this, "domwindowopened");
                this._observerService.removeObserver(this, "domwindowclosed");
                break;
            case "domwindowopened" :
                var window = subject.QueryInterface(Components.interfaces.nsIDOMWindow);
                this.watchWindow(window, true);
                break;
            case "domwindowclosed" :
                var window = subject.QueryInterface(Components.interfaces.nsIDOMWindow);
                this.watchWindow(window, false);
                break;
        }
    },

    IsPlaying: function() {
        var YMP = this.findYMP();
        if (!YMP) return false;

        return this.YMPIsPlayingInDoc(YMP.doc);
    },

    IsClosed: function() {
        return this._YMPList.length ? false : true;
    },

    YMPFoundInDoc: function(doc) {
        var scriptTags = doc.getElementsByTagName("script");
        var bYMPAlreadyInPage = false;
        for (var i=0; i<Math.min(scriptTags.length, 1000); i++) {
            if (!scriptTags[i].hasAttribute("src")) continue;
            if (!scriptTags[i].getAttribute("src").match(/mediaplayer.yahoo.com/)) continue;
            bYMPAlreadyInPage = true;
            break;
        };
        return bYMPAlreadyInPage;
    },
 
    YMPIsPlayingInDoc: function(doc) {
        var playerState = this.chromeToContentBridge.fireCommand(doc, "YAHOO.MediaPlayer.getPlayerState()"); 
        return (playerState == this._YMP_STATE_PLAYING) ? true : false;
    },

    findYMP : function() {
        var YMPDetails = null;
        // If last YMP is still playing than return it
        if (this._currentYMP && this._currentYMP.doc.defaultView) {
            // if YMP still found in last YMP tab than return it if no other YMP is playing
            if (this.YMPFoundInDoc(this._currentYMP.doc)) {
                YMPDetails = this._currentYMP;
            }
            if (this.YMPIsPlayingInDoc(this._currentYMP.doc)) return this._currentYMP; 
        }

        for (var i=0; i<this._YMPList.length; i++) {
            var currentYMP = this._YMPList[i];
            
            // return the first found YMP tab if no previous YMP tab found
            if (!YMPDetails) {
                YMPDetails = currentYMP;
            }

            if (this.YMPIsPlayingInDoc(currentYMP.doc)) {
                this._currentYMP = currentYMP;
                return currentYMP;
            }
        }

        this._currentYMP = YMPDetails;
        return YMPDetails;
    },

    initYMPList : function() {
        delete this._YMPList;
        this._YMPList = null;
        this._YMPList = new Array();

        var typeList = ['navigator:browser', 'foxytunes:minibrowser'];
        for (var i in typeList) {
            var windows = this._windowMediator.getEnumerator(typeList[i]);
            while (windows.hasMoreElements()) {
                var wantedWindow = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
                // we do not want popup windows (?)
                if (!wantedWindow.toolbar.visible) continue;

                this.watchWindow(wantedWindow, true);

                var tabbrowser = wantedWindow.getBrowser();
                var tabs = tabbrowser.mTabContainer.childNodes;
                for (var iTab=0; iTab < tabs.length; iTab++) {
                    var tab = tabs[iTab];
                    var browser = tabbrowser.getBrowserForTab(tab);
                    var doc = browser.contentDocument;
                    if (!doc) continue;

                    // Look for YMP 
                    if (!this.YMPFoundInDoc(doc)) continue;

                    this.addYMPToList(wantedWindow, tab, doc);              
                }
            }
        }
    },

    addYMPToList: function(window, tab, doc) {
        this.chromeToContentBridge.init(doc);

         var me = this;
         doc.defaultView.addEventListener("unload", function() { me.removeYMPFromList(doc); }  , true);

                    var playerDetails = new Object();
         playerDetails.window  = window;
         playerDetails.browser = window.getBrowser();
                    playerDetails.tab     = tab;
                    playerDetails.doc     = doc;

                    this._YMPList.push(playerDetails);
    },
                    
    removeYMPFromList: function(doc) {
        var tmpList = new Array();
        for (var i=0; i<this._YMPList.length; i++) {
            var currentYMP = this._YMPList[i];
            if (currentYMP.doc == doc) continue;
            tmpList.push(currentYMP);
                    }
        delete this._YMPList;
        this._YMPList = null;
        this._YMPList = tmpList;
        if (this._currentYMP && this._currentYMP.doc == doc) this._currentYMP = null;
    },

    watchWindow: function(window, bWatch) {
        if (bWatch) {
            window.addEventListener("DOMContentLoaded", this._domContentLoadedCallback, true);
        } else {
            window.removeEventListener("DOMContentLoaded", this._domContentLoadedCallback, true);
        }
    },

    onDOMContentLoad: function(event) {
        if (event.originalTarget.nodeName != "#document") { return; }
        var doc = event.originalTarget || document;
        if (!doc) return;
        
        var window = event.currentTarget;
        if (!window) return;
        
        var me = this;
        doc.addEventListener("FoxyTunesYMPInit", function(event) { me.onYMPInit(window, doc); }, true);
        doc.addEventListener("GooseInit", function(event) { me.onYMPInit(window, doc); }, true);

        // This part will be redundant when YMP will have FoxyTunesYMPInit event by default
        // Look for YMP 
        if (!this.YMPFoundInDoc(doc)) return;
        this.onYMPInit(window, doc);
    },

    onYMPInit: function(window, doc) {
        // Don't add doc if it's already on the list
        for (var i=0; i<this._YMPList.length; i++) {
            var currentYMP = this._YMPList[i];
            if (currentYMP.doc == doc) return;
        }

        var browser = window.getBrowser();
        var tab = this.getTabFromDoc(browser, doc);
        if (!tab) return;

        this.addYMPToList(window, tab, doc);  
    },

    getTabFromDoc: function(browser, doc) {
        var tabs = browser.tabContainer.childNodes;
        for(var i = 0; i < tabs.length; i++) {
          if (tabs[i].linkedBrowser.contentDocument == doc) {
            return tabs[i];
          }
        }
        return null;
    },

    getPlayerViewState: function(doc) {

        var rc = this.chromeToContentBridge.fireCommand(doc, "YAHOO.MediaPlayer.getPlayerViewState()");

        // Sanity
        if ((rc != 0) && (rc != 1)) rc = 0;
        

        return parseInt(rc);
    },

    setPlayerViewState: function(doc, bShow) {
        var state = bShow ? 1 : 0;
        this.chromeToContentBridge.fireCommand(doc, "YAHOO.MediaPlayer.setPlayerViewState(" + state + ")");
    },

    getQueueViewState: function(doc) {
        var rc = this.chromeToContentBridge.fireCommand(doc, "YAHOO.MediaPlayer.getQueueViewState()");

        // Sanity
        if ((rc != 0) && (rc != 1)) rc = 0;

        return parseInt(rc);
    },

    setQueueViewState: function(doc, bShow) {
        var state = bShow ? 1 : 0;
        this.chromeToContentBridge.fireCommand(doc, "YAHOO.MediaPlayer.setQueueViewState(" + state + ")");
    },
   
    getCurrentWindowAndTab : function() {
        var lastWindow = this._windowMediator.getMostRecentWindow('navigator:browser');
        if (!lastWindow) {
            lastWindow = this._windowMediator.getMostRecentWindow('foxytunes:minibrowser');
        }
        if (!lastWindow) return null;

        var lastTabbrowser = lastWindow.getBrowser();
        var lastTab = lastTabbrowser.selectedTab; 
        return [lastWindow, lastTab];
    },

    getMostRecentBrowserWindow : function() {
        var win = this._windowMediator.getMostRecentWindow('navigator:browser');
        if (win && win.toolbar.visible) {
            return win;
        }

        // else
        var windows = this._windowMediator.getEnumerator('navigator:browser', true);
        while (windows.hasMoreElements()) {
           win = windows.getNext();
           if (win.toolbar.visible) return win; 
        }

        return null;
    },

    openYMP: function() {
        const ympURL = 'http://www.foxytunes.com/mediaplayers/yahoo-media-player';
        var win = this.getMostRecentBrowserWindow();
        if (win) {
                var tabbrowser = win.getBrowser();

           		var newTab = tabbrowser.addTab(ympURL);
                tabbrowser.selectedTab = newTab;
        } else {
            var windowWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                                    .getService(Components.interfaces.nsIWindowWatcher);
            var array = Components.classes["@mozilla.org/supports-array;1"]
                                .createInstance(Components.interfaces.nsISupportsArray);
            var url = Components.classes["@mozilla.org/supports-string;1"]
                                .createInstance(Components.interfaces.nsISupportsString);
            url.data = ympURL;

            array.AppendElement(url);
            windowWatcher.openWindow(windowWatcher.activeWindow,
                                 "chrome://browser/content/browser.xul",
                                 "_blank",
                                 "chrome,all,dialog=no",
                                 array);
       }
    },

    printTrace : function(message) {
        dump(message + '\n');
    }

}
