// Copyright 2006 FoxyTunes
// All Rights Reserved

if ((typeof(FTJSLib.JS_LIB_LOADED) == 'boolean') &&
    (typeof(FT_LIB_LOADED) == 'boolean')) { 

// Languages Const:
const ENGLISH_LANG = 'English';
const WESTERN_EUROPEAN_LANG  = 'Western_European';
const JAPANESE_LANG  = 'Japanese';
const RUSSIAN_LANG  = 'Russian';
const HEBREW_LANG  = 'Hebrew';
const FRENCH_LANG  = 'French';
const PORTUGUESE_LANG  = 'Portuguese';
const GERMAN_LANG  = 'German';
const ARABIC_LANG  = 'Arabic';
const CHINESE_LANG  = 'Chinese';
const KOREAN_LANG  = 'Korean';
const EAST_ASIAN_LANG = 'EastAsian';
const UNKNOWN_LANG = 'Unknown';

// Class: LanguageDetector
function LanguageDetector() {

}

LanguageDetector.prototype = {
    
    detectLanguage: function(str) {
	var lang = null;
	var unknown = false;
	for (var i=0; i < str.length; i++) {
	    var charCode = str.charCodeAt(i);

	    if (charCode <= 127) {
		if (!lang) {
		    lang = ENGLISH_LANG;
		}
		continue;
	    } else if ( (charCode <= 591 ||  (charCode >= 7680 && charCode <= 7935) ) 
			&& (lang == null || lang == ENGLISH_LANG || lang == WESTERN_EUROPEAN_LANG ) ) {
		// we get in here in case that we have a WESTERN_EUROPEAN char code and we did not detect any other language Like EastAsian.
		// we must remeber that a East Asian string can contains chars that are Western like euro sing or any other. 
		lang = this.discoverWesternEuropeanLanguage(charCode);
		if (lang !=  WESTERN_EUROPEAN_LANG ) {
		    break;
		}
		continue;
	    } else if ( this.isEastAsian(charCode) ) {
		lang = EAST_ASIAN_LANG;
		 // we continue in order to check if it is (Chinese, Japanese, korean) because there is no overlap 
		// between EastAsian and specific Asian languages characters.
		continue;			 
	    } else if ( this.isJapaneseChar(charCode) ) {
		lang = JAPANESE_LANG;
		break;
	    } else if ( this.isChineseChar(charCode) ) {
		lang = CHINESE_LANG;
		break;
	    } else if ( this.isKoreanChar(charCode) ) {
		lang = KOREAN_LANG;
		break;
	    } else if ( this.isRussianChar(charCode) ) {
		lang = RUSSIAN_LANG; // Cyrillic 
		break;
	    } else if ( this.isHebrewChar(charCode) ) {
		lang = HEBREW_LANG; 
		break;
	    } else if ( this.isArabicChar(charCode) ) {
		lang = ARABIC_LANG; 
		break;
	    } else if ( this.isPunctuationChar(charCode) || this.isFullwidthLatinChar(charCode) ) {
		continue;
	    } else if ( this.isUnKnownLanguage(charCode)) {
		unknown = true;
		continue;
	    } else {
		lang = null;
		break;
	    }
	}
	
	if (lang == null && unknown) {
	    lang = UNKNOWN_LANG;
	}	

	return lang;
    },

    discoverWesternEuropeanLanguage: function(charCode) {
	var lang = WESTERN_EUROPEAN_LANG;
	switch (charCode) {
	    case 255:
	    case 193:
	    case 231:
	    case 199:
		lang = FRENCH_LANG;
		break;
	    case 223:
	    case 228:
	    case 196:
		lang = GERMAN_LANG;
		break;
	} 
	return lang;
    },

    isJapaneseChar: function(charCode) {
	if ( (charCode >= 12352 && charCode <= 12543)
		|| (charCode >= 12784 && charCode <= 12799) 
		|| (charCode >= 65381 && charCode <= 65439) ) {
		return true;
    	}
	return false;
    },

    isRussianChar: function(charCode) {
	if ( charCode >= 1024 && charCode <= 1327 ) { // Cyrillic 
		return true;
    	}
	return false;
    },

    isHebrewChar: function(charCode) {
	if ( charCode >= 1424 && charCode <= 1535 ) {  
		return true;
    	}
	return false;
    },

    isEastAsian: function(charCode) {
	// Unified CJK Ideographs :    Range 4E00  - 9FBF
	// CJK Ideographs Ext. A :     Range 3400  - 4DBF
	// CJK Ideographs Ext. B :     Range 20000 - 2A6DF
	// Compatibility Ideographs :  Range F900  - FAFF
	// Compatibility Ideo. Suppl : Range 2F800 - 2FA1F
	// Kanbun :                    Range 3190  - 319F
	if ( (charCode >= 19968 && charCode <= 40895) 
		|| (charCode >= 13312 && charCode <= 19903) 
		|| (charCode >= 131072 && charCode <= 173791) 
		|| (charCode >= 63744 && charCode <= 64255) 
		|| (charCode >= 194560 && charCode <= 195103) 
		|| (charCode >= 12688 && charCode <= 12703) ) {
		return true;
    	}
	return false;
    },
    
    isChineseChar: function(charCode) {
	// Bopomofo          : Range 3100 - 312F
	// Bopomofo Extended : Range 31A0 - 31BF
	if ( (charCode >= 12544 && charCode <= 1259)
		|| (charCode >= 12704 && charCode <= 12735) ) {
		return true;
    	}
	return false;
    },

    isKoreanChar: function(charCode) {
	// Hangul Syllables :          Range AC00 - D7AF
	// Hangul Jamo      :	       Range 1100 - 11FF
	// Hangul Compatibility Jamo : Range 3130 - 318F
	// Halfwidth Hangul variants : Range FFA0 - FFDC
	if ( (charCode >= 44032 && charCode <= 55215)
		|| (charCode >= 4352 && charCode <= 4607) 
		|| (charCode >= 12592 && charCode <= 12687) 
		|| (charCode >= 65440 && charCode <= 65500) ) {
		return true;
    	}
	return false;
    },

    isArabicChar: function(charCode) {
	// Arabic :                      Range 0600 - 06FF
	// Arabic Supplement :           Range 0750 - 077F
	// Arabic Presentation Forms-A : Range FB50 - FDFF
	// Arabic Presentation Forms-B : Range FE70 - FEFF
	if ( (charCode >= 1536 && charCode <= 1791)
		|| (charCode >= 1872 && charCode <= 1919) 
		|| (charCode >= 64336 && charCode <= 65023) 
		|| (charCode >= 65136 && charCode <= 65279) ) {
		return true;
    	}
	return false;
    },

    isPunctuationChar: function(charCode) {
	// General Punctuation      : Range 2000 - 206F
	// Supplement Punctuation   : Range 2E00 - 2E7F
	// CJK Punctuation          : Range 3000 - 303F
	// Verticl Forms            : Range FE10 - FE1F
	// Small Forms Variannts    : Range FE50 - FE6F
	// Number Forms             : Range 2150 - 218F
	// Specials                 : Range FFF0 - FFFF
	if ( (charCode >= 8192 && charCode <= 8303)
		|| (charCode >= 11776 && charCode <= 11903)
		|| (charCode >= 12288 && charCode <= 12351)
		|| (charCode >= 65040 && charCode <= 65055)
		|| (charCode >= 65104 && charCode <= 65135)
		|| (charCode >= 8528 && charCode <= 8591)
		|| (charCode >= 65520 && charCode <= 65535) ) {
		return true;
    	}
	return false;
    },

    isFullwidthLatinChar: function(charCode) {
	// Fullwidth Latin      : Range FF01 - FF60
	if ( (charCode >= 65281 && charCode <= 65376) ) {
		return true;
    	}
	return false;
    },

    isUnKnownLanguage: function(charCode) {
	if ( (charCode >= 8304 && charCode <= 11903)
		|| (charCode >= 64256 && charCode <= 64335)
		|| (charCode >= 65024 && charCode <= 65039)
		|| (charCode >= 65536 && charCode <= 66463)
		|| (charCode >= 118784 && charCode <= 119375)
		|| (charCode >= 119808 && charCode <= 120831) ) {
		return true;
    	}
	return false;
    },
}

} else { dump("Load Failure: languageDetector.js\n"); }
